var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { uploads } from '../modules/configs/uploads.js';
import { createBasicVirtualHost, isBasic } from './virtual-host/basic.js';
import { createProxy } from './virtual-host/apache.js';
import { pastDomains, writeLogs } from './virtual-host/logs.js';
import { setCommands, buildCommands } from '../modules/commands.js';
import sh from '../modules/sh.js';
// SSH
import { catchExec, connect, end, exec } from '../ssh.js';
import reboot from './steps/reboot.js';
import { downloads } from '../modules/configs/downloads.js';
const createSVPS = () => {
    let connected = false;
    let access;
    const createConnection = (access) => __awaiter(void 0, void 0, void 0, function* () {
        yield connect(access);
        /** Just checking the connection */
        yield catchExec('sleep 1');
        connected = true;
    });
    return class SVPS {
        constructor(options) {
            /**
             * Prepare and execute the command queue.
             */
            this.mount = (options) => __awaiter(this, void 0, void 0, function* () {
                try {
                    console.log(`\x1b[22m\x1b[36m\x1b[1m⦿ ${access.username}@${access.host}\x1b[0m`);
                    if (!connected)
                        yield createConnection(access);
                    /** Only connection */
                    if (!options)
                        return true;
                    const configs = options;
                    yield setCommands({
                        configs,
                        access,
                    });
                    /** Clean up */
                    yield exec('history -c', access);
                    if (configs.reboot)
                        yield reboot(access);
                    console.log('\x1b[0m');
                    console.log(`\x1b[0m\x1b[1m\x1b[32m✔︎ Success\x1b[0m\n`);
                    return true;
                }
                catch (error) {
                    if (error instanceof Error)
                        throw new Error(`\x1b[0m\x1b[1m\x1b[31m✖︎ ${error.message} \x1b[0m\n`);
                }
            });
            /**
             * Create the Virtual Hosts
             */
            this.createVirtualHosts = (
            /**
             * Required:
             *   - `apache` (to proxy the virtual ports to `80`)
             *
             * Optional:
             *   - `docker` (required to automatically create the **Basic Servers**)
             */
            virtualHosts) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const basicVirtualHosts = [];
                    const advancedVirtualHosts = [];
                    const invalidVirtualHosts = [];
                    const deployedVirtualHosts = [];
                    const commands = [
                        'echo "debconf debconf/frontend select Noninteractive" | sudo debconf-set-selections',
                        'sudo mkdir -p /var/containers/images /var/containers/compositions /var/containers/domains /var/containers/databases',
                        'sudo chmod 0755 /var/containers',
                        'sudo chmod 0700 /var/containers/images',
                        'sudo chmod 0700 /var/containers/compositions',
                        'sudo chmod 0750 /var/containers/databases',
                        'sudo chmod 0755 /var/containers/domains',
                    ];
                    if (!virtualHosts) {
                        console.log(`\n\x1b[0m\x1b[1m\x1b[32m✔︎ No domain has been set up\x1b[0m\n`);
                        return true;
                    }
                    /** Preparing Virtual Hosts */
                    virtualHosts.forEach((virtualHost) => {
                        if (typeof virtualHost === 'object' && !('server' in virtualHost))
                            advancedVirtualHosts.push(virtualHost);
                        else if (isBasic(virtualHost))
                            basicVirtualHosts.push(virtualHost);
                        else
                            invalidVirtualHosts.push(virtualHost);
                    });
                    /** Wrong Virtual Hosts */
                    if (invalidVirtualHosts.length > 0)
                        throw `Invalid Virtual Hosts: ${invalidVirtualHosts
                            .map((vh) => `\n  - ${vh.domain}`)
                            .join('')}`;
                    if (pastDomains.length > 0) {
                        console.log(`\n\x1b[22m\x1b[36m\x1b[1m▸ Skipping already created domains \x1b[2m\x1b[3m(./.svps/domains.json)\x1b[0m`);
                        console.log(pastDomains
                            .map((domain) => ` \x1b[36m⌙\x1b[0m ${domain}`)
                            .join('\n'));
                    }
                    /** Advanced (Manual) Virtual Hosts */
                    if (advancedVirtualHosts.length > 0) {
                        advancedVirtualHosts.forEach((virtualHost) => {
                            if (pastDomains.includes(virtualHost.domain))
                                return;
                            deployedVirtualHosts.push(virtualHost);
                            Object.assign(commands, [
                                ...commands,
                                `echo "${sh.startTitle}Proxy Port: ${virtualHost.domain} on port ${virtualHost.port}${sh.endTitle}"`,
                                ...createProxy(virtualHost),
                                'sudo systemctl reload apache2',
                                sh.done,
                            ]);
                        });
                        commands.push('sudo systemctl restart apache2');
                    }
                    /** Basic Virtual Hosts Servers */
                    if (basicVirtualHosts.length > 0) {
                        basicVirtualHosts.forEach((virtualHost) => {
                            if (pastDomains.includes(virtualHost.domain))
                                return;
                            deployedVirtualHosts.push(virtualHost);
                            Object.assign(commands, [
                                ...commands,
                                ...createBasicVirtualHost(virtualHost),
                            ]);
                        });
                        commands.push('sudo systemctl restart apache2');
                    }
                    if (deployedVirtualHosts.length === 0)
                        console.log(`\n\x1b[0m\x1b[1m\x1b[32m✔︎ No new domain has been set up\x1b[0m`);
                    else {
                        console.log(`\n\x1b[22m\x1b[36m\x1b[1m⦿ Setting up Virtual Hosts: ${access.username}@${access.host}\x1b[0m`);
                        if (!connected)
                            yield createConnection(access);
                        for (const command of commands)
                            yield catchExec(command);
                        yield catchExec('history -c');
                        writeLogs(deployedVirtualHosts);
                    }
                    console.log(`\n\x1b[0m\x1b[1m\x1b[32m✔︎ Success\x1b[0m\n`);
                    return true;
                }
                catch (error) {
                    if (error instanceof Error)
                        throw new Error(`\x1b[0m\x1b[1m\x1b[31m✖︎ ${error.message} \x1b[0m\n`);
                }
            });
            /**
             * Mount and execute your commands
             *
             * ---
             *
             * Special commands:
             * - `['--reboot']`
             * - `['--restart-ssh']`
             * - `['--catch _your_command_']`
             */
            this.commands = (commandsQueue) => __awaiter(this, void 0, void 0, function* () {
                try {
                    console.log(`\x1b[22m\x1b[36m\x1b[1m⦿ ${access.username}@${access.host}\x1b[0m`);
                    if (!connected)
                        yield createConnection(access);
                    const commands = [
                        `echo "${sh.startTitle}Personal commands${sh.endTitle}"`,
                        ...commandsQueue,
                        sh.done,
                    ];
                    yield buildCommands({ commands, access });
                    /** Clean up */
                    yield exec('history -c', access);
                    console.log('\x1b[0m');
                    return true;
                }
                catch (error) {
                    if (error instanceof Error)
                        throw new Error(`\x1b[0m\x1b[1m\x1b[31m✖︎ ${error.message} \x1b[0m\n`);
                }
            });
            /**
             * Upload files and directories.
             */
            this.upload = (paths) => __awaiter(this, void 0, void 0, function* () {
                try {
                    console.log(`\x1b[22m\x1b[36m\x1b[1m⦿ ${access.username}@${access.host}\x1b[0m`);
                    if (!connected)
                        yield createConnection(access);
                    /** Uploading files and directories */
                    yield uploads(paths);
                    console.log(`\x1b[0m\x1b[1m\x1b[32m✔︎ Success\x1b[0m\n`);
                    return true;
                }
                catch (error) {
                    if (error instanceof Error)
                        throw new Error(`\x1b[0m\x1b[1m\x1b[31m✖︎ ${error.message} \x1b[0m\n`);
                }
            });
            /**
             * Download files.
             */
            this.download = (paths) => __awaiter(this, void 0, void 0, function* () {
                try {
                    console.log(`\x1b[22m\x1b[36m\x1b[1m⦿ ${access.username}@${access.host}\x1b[0m`);
                    if (!connected)
                        yield createConnection(access);
                    /** Uploading files and directories */
                    yield downloads(paths);
                    console.log(`\x1b[0m\x1b[1m\x1b[32m✔︎ Success\x1b[0m\n`);
                    return true;
                }
                catch (error) {
                    if (error instanceof Error)
                        throw new Error(`\x1b[0m\x1b[1m\x1b[31m✖︎ ${error.message} \x1b[0m\n`);
                }
            });
            this.end = end;
            access = options.access;
        }
    };
};
export const SVPS = createSVPS();
